/*
************************************************************************
*
*   ExDrawobj.c - AddDrawobj and MoveDrawobj commands
*
*   Copyright (c) 1995
*
*   ETH Zuerich
*   Institut fuer Molekularbiologie und Biophysik
*   ETH-Hoenggerberg
*   CH-8093 Zuerich
*
*   SPECTROSPIN AG
*   Industriestr. 26
*   CH-8117 Faellanden
*
*   All Rights Reserved
*
*   Date of last modification : 95/06/30
*   Pathname of SCCS file     : /sgiext/molmol/src/cmdprim/SCCS/s.ExDrawobj.c
*   SCCS identification       : 1.3
*
************************************************************************
*/

#include <cmd_prim.h>

#include <stdio.h>

#include <arg.h>
#include <data_hand.h>
#include <data_sel.h>
#include <prim_hand.h>
#include <graph_draw.h>

#define ARG_NUM 2
#define TYPE_NUM 3

typedef struct {
  int ind;
  Vec3 v;
} MoveData;

static int CurrType = 0, CurrDim = 1;

ErrCode
ExAddDrawobj(char *cmd)
{
  ArgDescr arg[ARG_NUM];
  EnumEntryDescr enum1Entry[TYPE_NUM];
  EnumEntryDescr enum2Entry[2];
  ErrCode errCode;
  DhMolP molP;
  PrimObjP primP;
  Vec3 x, dx;

  arg[0].type = AT_ENUM;
  arg[1].type = AT_ENUM;

  ArgInit(arg, ARG_NUM);

  enum1Entry[0].str = "line";
  enum1Entry[0].onOff = FALSE;
  enum1Entry[1].str = "rect";
  enum1Entry[1].onOff = FALSE;
  enum1Entry[2].str = "circle";
  enum1Entry[2].onOff = FALSE;

  enum1Entry[CurrType].onOff = TRUE;

  arg[0].prompt = "Type";
  arg[0].u.enumD.entryP = enum1Entry;
  arg[0].u.enumD.n = TYPE_NUM;
  arg[0].v.intVal = CurrType;

  enum2Entry[0].str = "2D";
  enum2Entry[0].onOff = FALSE;
  enum2Entry[1].str = "3D";
  enum2Entry[1].onOff = FALSE;

  enum2Entry[CurrDim].onOff = TRUE;

  arg[1].prompt = "Dim";
  arg[1].u.enumD.entryP = enum2Entry;
  arg[1].u.enumD.n = 2;
  arg[1].v.intVal = CurrDim;

  errCode = ArgGet(arg, ARG_NUM);
  if (errCode != EC_OK) {
    ArgCleanup(arg, ARG_NUM);
    return errCode;
  }

  CurrType = arg[0].v.intVal;
  CurrDim = arg[1].v.intVal;

  ArgCleanup(arg, ARG_NUM);

  if (CurrDim == 0) {
    primP = PrimNew(PT_DRAWOBJ, NULL);
    PrimSetDrawobjType(primP, CurrType);

    Vec3Zero(x);
    dx[0] = -0.2;
    dx[1] = -0.9;
    dx[2] = 0.0;
    PrimSetPoint(primP, 0, x, dx);
    dx[0] = 0.2;
    dx[1] = -0.7;
    dx[2] = 0.0;
    PrimSetPoint(primP, 1, x, dx);
  } else {
    if (SelMolGet(&molP, 1) != 1) {
      CipSetError("exactly 1 molecule must be selected");
      return EC_ERROR;
    }

    primP = PrimNew(PT_DRAWOBJ, molP);
    PrimSetDrawobjType(primP, CurrType);

    DhCalcCenter(x, molP);
    Vec3Zero(dx);
    PrimSetPoint(primP, 0, x, dx);
    dx[0] = 0.4;
    dx[1] = 0.2;
    PrimSetPoint(primP, 1, x, dx);
  }
  
  GraphRedrawNeeded();

  return EC_OK;
}

static void
moveObj(PrimObjP primP, void *clientData)
{
  MoveData *dataP = clientData;
  Vec3 x, dx;

  PrimGetPoint(primP, dataP->ind, x, dx);
  Vec3Add(dx, dataP->v);
  PrimSetPoint(primP, dataP->ind, x, dx);
}

#undef ARG_NUM
#define ARG_NUM 4

ErrCode
ExMoveDrawobj(char *cmd)
{
  ArgDescr arg[ARG_NUM];
  ErrCode errCode;
  MoveData data;

  arg[0].type = AT_INT;
  arg[1].type = AT_DOUBLE;
  arg[2].type = AT_DOUBLE;
  arg[3].type = AT_DOUBLE;

  ArgInit(arg, ARG_NUM);

  arg[0].prompt = "Point Index";
  arg[0].v.intVal = 0;

  arg[1].prompt = "Move x";
  arg[1].v.doubleVal = 0.0;

  arg[2].prompt = "Move y";
  arg[2].v.doubleVal = 0.0;

  arg[3].prompt = "Move z";
  arg[3].v.doubleVal = 0.0;

  errCode = ArgGet(arg, ARG_NUM);
  if (errCode != EC_OK) {
    ArgCleanup(arg, ARG_NUM);
    return errCode;
  }

  data.ind = arg[0].v.intVal;
  data.v[0] = arg[1].v.doubleVal;
  data.v[1] = arg[2].v.doubleVal;
  data.v[2] = arg[3].v.doubleVal;

  ArgCleanup(arg, ARG_NUM);

  PrimApply(PT_DRAWOBJ, PropGetRef(PROP_SELECTED, FALSE), moveObj, &data);

  GraphRedrawNeeded();

  return EC_OK;
}
